package unicrawler.crawler;

import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import unicrawler.CrawlerEnvironment;
import unicrawler.config.CrawlerConfig;

public class CrawlerMonitor implements Runnable {

	private CrawlerEnvironment mEnvironment;
	private ScheduledThreadPoolExecutor mPool;
	private CrawlerWorkerCounter mCounter;
	private CrawlerConfig mConfig;
	
	private Logger logger = Logger.getLogger(CrawlerMonitor.class.getName());
	
	@Override
	public void run() {
		try {
			do {
				if (this.mConfig.getThreadDebug()) { 
					logger.debug(String.format("\r\nPool size: %d\r\nActive Crawlers: %d\r\nCompleted: %d",
							this.mPool.getCorePoolSize(),
							this.mCounter.getNumOfCrawlers(),
							this.mPool.getCompletedTaskCount()
						)
					);
				}
				
				Thread.sleep(1000);	// 10seconds, this blocks at the end of execution though
				
			} while (this.mCounter.getNumOfCrawlers() != 0);
			
		} catch (InterruptedException e) {
			logger.error("Crawler monitor thread died!");
			logger.error(e);
			
		}
		finally {			
			logger.info("Discovered a total of " + this.mEnvironment.getFrontier().getCrawledUrlCount() + " page(s)");
			logger.info("Successfully crawled " + this.mEnvironment.getFrontier().getSuccessfulCrawlCount() + "/" + this.mEnvironment.getFrontier().getCrawledUrlCount() + " of these pages.");
			logger.info("Redirects: " + this.mEnvironment.getRedirectedCrawlCount() + ", Errors: " + this.mEnvironment.getResponseErrorCount() + ", Skipped: " + this.mEnvironment.getSkippedCount());
						
			System.out.println("");
			
			long duration = System.currentTimeMillis() - this.mEnvironment.getStartTime();
			
			logger.info("Crawl finished in " + String.format("%02dmins, %02dsecs and %02dms", TimeUnit.MILLISECONDS.toMinutes(duration), TimeUnit.MILLISECONDS.toSeconds(duration), TimeUnit.MILLISECONDS.toMillis(duration)));
			logger.info("Crawler ThreadPool is shutting down now...");

			// Stop the frontier thread from processing the queue and shutdown the ThreadPool daemon
			this.mEnvironment.getFrontier().stop();
			this.mPool.shutdown();
		}
	}

	public CrawlerMonitor(CrawlerEnvironment _env) {
		this.mEnvironment = _env;
		
		this.mConfig = _env.getConfig();
		this.mPool = _env.getThreadpool();
		this.mCounter = _env.getThreadpoolCounter();
	}
}
