package unicrawler.parser;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import unicrawler.utils.UrlUtils;

public class HyperlinkParser implements IParser {

	private String mContent;
	private String mParentUrl;
	private String mPageCharset;
	
	private final String mRegex = "(?i)(?s)<\\s*?a.*?href=\"(.*?)\".*?>";
	
	public HyperlinkParser(String _content, String _parentURL, String _charset) {
		/** parentUrl for URL normalization! */
		if (_content.trim().equals("") || _content == null) {
			throw new IllegalArgumentException("Content for HyperlinkParser cannot be null!");
		}
		
		if (_parentURL.trim().equals("") || _parentURL == null) {
			throw new IllegalArgumentException("Parent URL cannot be null!");
		}
		
		if (_charset == null) {
			_charset = "UTF-8";
		}
		
		this.mContent = _content;
		this.mParentUrl = _parentURL;
		this.mPageCharset = _charset;
	}
	
	@Override
	public List<String> harvest() {
		List<String> hyperlinks = new ArrayList<String>();

		Pattern pattern = Pattern.compile(this.mRegex);
		Matcher matcher = pattern.matcher(this.mContent);		
		
		while (matcher.find()) {
			// Normalize the links found
			String link = matcher.group(1);
			
			// This is a pretty lame attempt at URL canonicalization by specifying charset set by downloader headers
			try {
				link = URLDecoder.decode(link, this.mPageCharset);
			}
			catch (UnsupportedEncodingException ex) {
				// if this happens, there's no fallback
			}
			
			// attempt to normalize url
			link = UrlUtils.resolveUrl(this.mParentUrl, link);
			
			// Check we have proper protocol, url normalization doesn't check for rss://, feed:// etc
			String identifier = link.substring(0, 5);
			if ( ! identifier.equalsIgnoreCase("http:") ) {
				continue;
			}
			
			hyperlinks.add(link);
		}
		
		return hyperlinks;
	}

}
